#!/usr/bin/perl

#                     MySQL Quota Daemon
#
# Copyright (c) 2006 Remigiusz "lRem" Modrzejewski <http://lrem.net/>
# For more info see http://lrem.net/mysql_quota.xhtml
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

use strict;
use DBI;
use CGI;
# DoS prevention.
$CGI::POST_MAX = 5*1024;
$CGI::DISABLE_UPLOADS = 1;

my $conf =
{
	host => 'localhost',
	user => 'root',
	pass => '',
	db => 'quotadb'
};

sub readconf
{
	return 0 unless open(CONFFILE, $_[0]);
	while(my $input=<CONFFILE>)
	{
		$input=~s/[^\\]#.*//;
		(my $option, my $value) = split(/\t/, $input, 2);
		$option=~s/\s*$//;
		$option=~s/^\s*//;
		$value=~s/\s*$//;
		$value=~s/^\s*//;
		$conf->{$option}=$value;
	}
	close(CONFFILE);

	return 1;
}

#INSTALL_CONFPATH
elsif(-e '/etc/mysql_quota')
{
	&readconf('/etc/mysql_quota');
}
else
{
	die "Couldn't read config.\n";
}

my $dbh = DBI->connect(
	'DBI:mysql:'.$conf->{'db'}.':'.$conf->{'host'},
	$conf->{'user'},
	$conf->{'pass'},
	{AutoCommit => 1}
) or die("Could not connect to database.\n");

sub list_screen
{
	my @rows;
	my $ref = $dbh->selectall_arrayref('SELECT `db`, `limit`, `exceeded` FROM quota');

	foreach my $rowref (@{$ref})
	{
		push @rows,
		CGI::td([
			$rowref->[0],
			$rowref->[1],
			$rowref->[2],
			'<a href='.CGI::url().'?db='.$rowref->[0].'>Edit</a>'
		]);
	}

	print CGI::table(
		CGI::Tr(
			[
			CGI::th(['Database name', 'Limit', 'Exceeded', 'Action']),
			CGI::Tr(\@rows)
			]
		)
	), CGI::a({href=>CGI::url().'?db=new_name'}, 'Add new entry');
}

sub edit_screen
{
	my $limit = $dbh->selectrow_array('SELECT `limit` FROM quota WHERE `db`='.
		$dbh->quote(CGI::param('db')));
	
	print CGI::start_form(),
	'Database name:', CGI::textfield('db', CGI::param('db')), CGI::br(),
	'Size limit (bytes):', CGI::textfield('limit', $limit), CGI::br(),
	CGI::submit(),
	CGI::end_form();
}

sub apply_screen
{
	if($dbh->do('REPLACE INTO quota (`db`, `limit`) VALUES ('.
				$dbh->quote(CGI::param('db')).','.
				$dbh->quote(CGI::param('limit')).')'
			))
	{
		print 'Database updated successfuly.', CGI::br(),
		CGI::a({href=>CGI::url()}, 'Back to the list.');
	}
	else
	{
		print 'An error occured:', $dbh->errstr, CGI::br(),
		CGI::a({href=>CGI::url()}, 'Back to the list.');
	}

}

print CGI::header({-charset => $conf->{'charset'} }),
	CGI::start_html({title=>'MySQL Quota Daemon Frontend', charset=>$conf->{'charset'}});
	
if(CGI::param('db'))
{
	if(CGI::param('limit'))
	{
		&apply_screen;
	}
	else
	{
		&edit_screen;
	}
}
else
{
	&list_screen;
}

